// EPROM Generation program for 405 line test card signal
// V1.0 Copyright (c) 2000-2002 J. Evans
// This software is supplied as-is and with no warrenty expressed or implied.
// Feel free to modify or use this software for any purpose, however if you re-distribute
// this software source you must include an acknowledgment of the original software and
// also distribute the source for free.
// TTFN,
//   Jon    www.TheValvePage.com

#include <stdio.h>
#include <stdlib.h>
#include <process.h>

#include "defs.h"

#define OUTPUT_FILENAME argv[1]
#define INPUT_FILENAME  argv[2]

typedef
enum
{
MODE_NONE=0,
MODE_TEST0,
MODE_TEST1,
MODE_BLACK,
MODE_HATCH,
MODE_FILE
} PATTERN ;

#define BLANK_LINE      9999 /* Value passed as 'y' param for a blank line */

#define FIRST_HALF_LINE	1
#define LAST_HALF_LINE  2
#define WHOLE_LINE      3


#define WHITE_PIXEL 127
#define BLACK_PIXEL 0


// The following few routines generate specific small chuncks
// of the waveform

void WriteFramePulse( FILE *op )
{
WORD t ;

   for( t = 0 ; t < FRAME_PULSE_LOW_TIME ; t++ )
   {
      fprintf( op, "%c", 0x83 ) ;
   }
   for( t = 0 ; t < FRAME_PULSE_HIGH_TIME ; t++ )
   {
      fprintf( op, "%c", 0x82 ) ;
   }
}

void WriteLinePulse( FILE *op )
{
WORD     t ;

   for( t = 0 ; t < LINE_PULSE_TIME ; t++ )
   {
      fprintf( op, "%c", 0x81 ) ;
   }
}

void WriteFrontPorch( FILE *op )
{
WORD	t ;

   for( t = 0 ; t < FRONT_PORCH_TIME ; t++ )
   {
      fprintf( op, "%c", BLACK_PIXEL ) ;
   }
}

void WriteBackPorch( FILE *op )
{
WORD t ;

   for( t = 0 ; t < BACK_PORCH_TIME ; t++ )
   {
      fprintf( op, "%c", BLACK_PIXEL ) ;
   }
}


// This function builds up a buffer containing the video information
// for a line. This could be read from a .BMP file, or alternatively we
// can create a few simple pre-defined patterns for test purposes.

// NOTE This creates just the video information without any sync pulses and
// that the specified line number is the actual picture line i.e. non-interlaced.

void GetLine( BYTE *pBuffer, WORD nLine, PATTERN Pattern )
{
WORD i ;

    if( BLANK_LINE == nLine )
    {
        for( i = 0 ; i < PIX_PER_LINE ; i++ )
        {
            *pBuffer++ = 0 ;
        }
    }

    else switch( Pattern )
    {
        case MODE_TEST0 : // Simple ramp
                for( i = 0 ; i < PIX_PER_LINE ; i++ )
                {
                    *pBuffer++ = (BYTE)(i%128) ;
                }
                break ;

        case MODE_TEST1 : // Half line of 300KHz, Half line of 3MHz for frequency reponse testing
                for( i = 0 ; i < (PIX_PER_LINE/2) ; i++ )
                {
                    *pBuffer++ = (i%20 < 10) * WHITE_PIXEL ;
                }
                for( i = (PIX_PER_LINE/2) ; i < PIX_PER_LINE ; i++ )
                {
                    *pBuffer++ = (i & 1) * WHITE_PIXEL ;
                }
                break ;

        case MODE_BLACK :
                for( i = 0 ; i < PIX_PER_LINE ; i++ )
                {
                    *pBuffer++ = BLACK_PIXEL ;
                }
                break ;

        case MODE_HATCH :
                if( nLine % 16 ) // every 16th line is white
                {
                    for( i = 0 ; i < PIX_PER_LINE ; i++ )
                    {
                        *pBuffer++ = WHITE_PIXEL ;
                    }
                }
                else // else repeat one white pixel plus 15 black pixels
                {
                    for( i = 0 ; i < PIX_PER_LINE ; i++ )
                    {
                        if( 0 == i%16 )
                        {
                           *pBuffer++ = WHITE_PIXEL ;
                        }
                        else
                        {
                           *pBuffer++ = BLACK_PIXEL ;
                        }
                    }
                }
                break ;

        case MODE_FILE :
                ReadBitmapFile( nLine, pBuffer ) ;
                break ;

        default :
                break ;
    }
}



// Write one picture line to the output file. This involves taking the
// writing the sync pulses and including the various porches and video
// data.

void WriteLine( FILE *op, WORD nLine, WORD Portion, PATTERN Pattern )
{
BYTE LineBuffer[PIX_PER_LINE] ;
WORD x ;

   GetLine( LineBuffer, nLine, Pattern ) ;

   if( (Portion == FIRST_HALF_LINE) || (Portion == WHOLE_LINE) )
   {
      WriteLinePulse( op ) ;

      WriteBackPorch( op ) ;

      for( x = 0 ; x < HALF_PIC_LINE ; x++ )
      {
         fprintf( op, "%c", (LineBuffer[x] / 2) ) ;
      }
   }

   if( (Portion == LAST_HALF_LINE) || (Portion == WHOLE_LINE) )
   {
      for( x = HALF_PIC_LINE ; x < PIX_PER_LINE ; x++ )
      {
         fprintf( op, "%c", (LineBuffer[x] / 2) ) ;
      }

      WriteFrontPorch( op ) ;
   }
}


// This is the main routine that writes one complete EPROM image.
// NOTE : The line numbers are the video signal line numbers, not
// the picture line number.

void WriteEPROMImage( FILE *op, PATTERN Pattern )
{
WORD Line ;

	/*** Lines 0..3 contain pairs of frame pulses ***/

      for( Line = 0 ; Line <= 3 ; Line++ )
      {
          WriteFramePulse( op ) ;
          WriteFramePulse( op ) ;
      }

	/*** Lines 4 .. 13 contain black signal ***/

      for( Line = 4 ; Line <= 13 ; Line++ )
      {
          WriteLine( op, BLANK_LINE, WHOLE_LINE, Pattern ) ;
      }

	/***    Lines 14 .. 201 Even picture lines   ***/
	/*** Line  14 is physical picture line   0   ***/
	/*** Line  15 is physical picture line   2   ***/
	/*** Line 'n' is physical picture line 2n-28 ***/

      for( Line = 14 ; Line <= 201 ; Line ++ )
      {
          WriteLine( op, (WORD)(2*Line - 28), WHOLE_LINE, Pattern ) ;
      }

	/*** Line 202 consists of half of physical picture ***/
	/***      line 376 followed by a frame pulse.      ***/

      WriteLine( op, 376, FIRST_HALF_LINE, Pattern ) ;
      WriteFramePulse( op ) ;

	/*** Line 203..205 consist of pairs of frame pulses ***/

      for( Line = 203 ; Line <= 205 ; Line++ )
      {
         WriteFramePulse( op ) ;
         WriteFramePulse( op ) ;
      }

	/*** Line 206 consists of a single frame pulse ***/
	/***    with remainder of line set to black    ***/

      WriteFramePulse( op ) ;
      WriteLine( op, BLANK_LINE, LAST_HALF_LINE, Pattern ) ;

	/*** Lines 207..215 are blank lines ***/

      for( Line = 207 ; Line <= 215 ; Line++ )
      {
          WriteLine( op, BLANK_LINE, WHOLE_LINE, Pattern ) ;
      }

	/*** Line 216 consists of half a blank line followed ***/
	/***  by the second half of physical picture line 1  ***/

      WriteLine( op , BLANK_LINE, FIRST_HALF_LINE, Pattern ) ;
      WriteLine( op, 0, LAST_HALF_LINE, Pattern ) ;   // [JE : OOPS ! Originally missed this !!]

	/*** Lines 217..404 consist of picture data   ***/
	/*** Line 217 is physical picture line   3    ***/
	/*** Line 218 is physical picture line   5    ***/
	/*** Line 'n' is physical picture line 2n-431 ***/

      for( Line = 217 ; Line <= 404 ; Line++ ) // [JE Deliberate mistake ... twice !]
      {
          WriteLine( op, (WORD)(2*Line-433), WHOLE_LINE, Pattern ) ;
      }

	/*** Finally, a single pixel used to reset sequence counter ***/

      fprintf( op, "%c", 0x84 ) ;
}

// The main program entry point. Decodes any command line parameters.

void main( int argc, char *argv[] )
{
PATTERN Pattern = MODE_NONE ;
FILE	*op = NULL ;

   if( argc != 3 )
   {
     printf("USAGE :-\n") ;
     /// insert help stuff
   }
   else if( (op=fopen(OUTPUT_FILENAME, "wb")) == NULL )
   {
      printf( "Error opening output file %s\n", OUTPUT_FILENAME ) ;
   }

   else if( OpenBitmapFile( INPUT_FILENAME ) )
   {
       Pattern = MODE_FILE ;
   }

   else if( INPUT_FILENAME[0] == '0' )
   {
      Pattern = MODE_TEST0 ;
      printf("Generating test PROM 0\n" ) ;
   }

   else if( INPUT_FILENAME[0] == '1' )
   {
       Pattern = MODE_TEST1 ;
       printf("Generating test PROM 1\n" ) ;
   }

   else if( INPUT_FILENAME[0] == 'B' )
   {
       Pattern = MODE_BLACK ;
       printf("Generating black level test PROM\n") ;
   }

   else if( INPUT_FILENAME[0] == 'H' )
   {
       Pattern = MODE_HATCH ;
       printf("Generating black level test PROM\n") ;
   }


    /* Write image file */

   if( Pattern != MODE_NONE )
   {
       WriteEPROMImage( op, Pattern ) ;
   }


    /* Close the various files */

   if( MODE_FILE == Pattern )
   {
      CloseBitmapFile() ;
   }

   if( op != NULL )
   {
       fclose( op ) ;
   }
}
